// Copyright (C) 2023 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:flutter/material.dart';
import 'package:wyatt_ui_components/src/domain/entities/text_inputs/text_input_style.dart';

class TextInputThemeExtension extends ThemeExtension<TextInputThemeExtension> {
  const TextInputThemeExtension({
    this.normalStyle,
    this.focusedStyle,
    this.invalidStyle,
    this.disabledStyle,
  });

  /// The default style for TextInputs.
  final TextInputStyle? normalStyle;

  /// The style for TextInputs when they are focused.
  final TextInputStyle? focusedStyle;

  /// The style for TextInputs when they are invalid.
  final TextInputStyle? invalidStyle;

  /// The style for TextInputs when they are disabled.
  final TextInputStyle? disabledStyle;

  @override
  ThemeExtension<TextInputThemeExtension> lerp(
    covariant ThemeExtension<TextInputThemeExtension>? other,
    double t,
  ) {
    if (other is! TextInputThemeExtension) {
      return this;
    }
    return TextInputThemeExtension(
      normalStyle: TextInputStyle.lerp(normalStyle, other.normalStyle, t),
      focusedStyle: TextInputStyle.lerp(focusedStyle, other.focusedStyle, t),
      disabledStyle: TextInputStyle.lerp(disabledStyle, other.disabledStyle, t),
      invalidStyle: TextInputStyle.lerp(invalidStyle, other.invalidStyle, t),
    );
  }

  @override
  ThemeExtension<TextInputThemeExtension> copyWith({
    TextInputStyle? normalStyle,
    TextInputStyle? focusedStyle,
    TextInputStyle? disabledStyle,
    TextInputStyle? invalidStyle,
  }) =>
      TextInputThemeExtension(
        normalStyle: normalStyle ?? this.normalStyle,
        focusedStyle: focusedStyle ?? this.focusedStyle,
        disabledStyle: disabledStyle ?? this.disabledStyle,
        invalidStyle: invalidStyle ?? this.invalidStyle,
      );
}
